#include "elementCell.h"

#include <iostream>

#include "mathematicalTools.h"
#include "mixedTools.h"


elementCell::elementCell()  : abstractElement() 
    {
      setDefaultValues();
      setDefaults();
      elementName_ = nomdElements("cell");
      nbParam_ = elementName_.getElementNbParameters();
      parametersString_ = new string[nbParam_+1];
      //      parmelaOk_ = true;
        registerAcceptableSoftware(nomDeLogiciel::parmela, TBoolOk);
    }

void elementCell::setDefaultValues()
{
    defaultLabel_ = "cell";
    initialPhaseDef_ = 0.0;
    acceleratingFieldDef_ = 1.0;
    acceleratingShapeFileDef_ = "xxxxxxx";
    focusingMagFileDef_ = "xxxxxx";
    offsetMagDef_ = 0.0;
    scaleFactorDef_ = 1.0;
}

void elementCell::setDefaults()
{
    label_ = defaultLabel_;
    initialPhase_ = initialPhaseDef_;
    acceleratingField_ = acceleratingFieldDef_;
    acceleratingShapeFile_ = acceleratingShapeFileDef_;
    focusingMagFile_ = focusingMagFileDef_;
    offsetMag_ = offsetMagDef_;
    scaleFactor_ = scaleFactorDef_;
  }


  string* elementCell::getParametersString() const 
  {
   int compteur = -1;
    parametersString_[++compteur] = mixedTools::intToString(nbParam_);
  parametersString_[++compteur] = label_;
    parametersString_[++compteur] = mixedTools::doubleToString(lenghtElem_);
    parametersString_[++compteur] = mixedTools::doubleToString(aperture_);
    parametersString_[++compteur] = mixedTools::doubleToString(initialPhase_);
    parametersString_[++compteur] = mixedTools::doubleToString( phaseStepMax_);
    parametersString_[++compteur] = mixedTools::doubleToString(acceleratingField_);
    parametersString_[++compteur] = acceleratingShapeFile_;
    parametersString_[++compteur] = focusingMagFile_;
    parametersString_[++compteur] = mixedTools::doubleToString(offsetMag_);
    parametersString_[++compteur] = mixedTools::doubleToString(scaleFactor_);
   if ( compteur != nbParam_ ) {
     cerr << " elementCell::getParametersString() : ERROR nr pf parameters dosnt match " << endl;
     return NULL;
   }
    return parametersString_;
  }


void elementCell::setParametersString(string* param)
  {
    if ( param == NULL )
      {
	cerr << "  elementCell::setParametersString parameters empty parameter set";
	return;
      }
   int compteur = -1;
    int nbparam = atoi(param[++compteur].c_str());
    if ( nbparam != nbParam_ ) 
      {
	cerr << "  elementCell::setParametersString parameters do not match for a CELL";
	return;
      }

    label_ = param[++compteur];
    lenghtElem_ = atof(param[++compteur].c_str());
    aperture_ = atof(param[++compteur].c_str());
    initialPhase_ = atof(param[++compteur].c_str());
    phaseStepMax_ = atof(param[++compteur].c_str());
    acceleratingField_ = atof(param[++compteur].c_str());
    acceleratingShapeFile_ = param[++compteur];
    focusingMagFile_ = param[++compteur];
    offsetMag_ = atof(param[++compteur].c_str());
    scaleFactor_ = atof(param[++compteur].c_str());
  }

string elementCell::parmelaOutputFlow() const
  {
    ostringstream sortie;
    // on prend les troncatures tmax et rmax à 3 sigmas 
    sortie << "CELL /l=" << lenghtElem_ << "  /aper=" << aperture_ << endl;
    sortie << "  /iout=1  /phi0=" << initialPhase_ << " /E0=" << acceleratingField_ << endl;
    sortie << " /nc=1 /dwtmax=" << phaseStepMax_ << " /sym=-1" << endl;
    sortie << "CFIELD 1" << endl;
    sortie << acceleratingShapeFile_ << endl;
    if ( focusingMagFile_ != "") {
      sortie << "POISSON /zoff=" << offsetMag_ << " /rmult=" << scaleFactor_ << endl;
      sortie << focusingMagFile_ << endl;
    }

    return sortie.str();
  }


string elementCell::FileOutputFlow() const
  {
    ostringstream sortie;
    //    sortie << elementName_.getElementType()  << endl;
    sortie << elementName_.getElementLabel() << endl;
    sortie  << label_ << endl;
    sortie << lenghtElem_ << " " << aperture_ << endl;
    sortie << initialPhase_ << "  " << acceleratingField_ << endl;
    sortie << phaseStepMax_ << endl;
    sortie << acceleratingShapeFile_ << endl;
    sortie << focusingMagFile_ << endl;
    sortie << offsetMag_ << " " << scaleFactor_ << endl;
    return sortie.str();
  }

void elementCell::FileInput(ifstream& ifs)
{
  ifs >> label_;
  ifs >> lenghtElem_ >>  aperture_;
  ifs >> initialPhase_ >> acceleratingField_;
  ifs >> phaseStepMax_;
  ifs >> acceleratingShapeFile_;
  ifs >> focusingMagFile_;
  ifs >> offsetMag_ >> scaleFactor_;
}


string elementCell::print() {
    string txt = "";


    txt += label_;
    txt += "\nlength (cm) : ";
    txt += mixedTools::doubleToString(lenghtElem_);
    txt += "\naperture (cm) : ";
    txt += mixedTools::doubleToString(aperture_);
    txt += "\ninitial phase (deg.) : ";
    txt += mixedTools::doubleToString(initialPhase_);
    txt += "\nmax phase step (deg.) : ";
    txt += mixedTools::doubleToString(phaseStepMax_);
    txt += "\nmax accelerating field (MV/m) : ";
    txt += mixedTools::doubleToString(acceleratingField_);
    txt += "\nname of file for field shape : ";
    txt += acceleratingShapeFile_;
    txt += "\nname of file for focusing mag. field : ";
    txt += focusingMagFile_;
    txt += "\n z-offset for mag. field : ";
    txt += mixedTools::doubleToString(offsetMag_);
    txt += "\n scaling factor for mag. field : ";
    txt += mixedTools::doubleToString(scaleFactor_);
    return txt;
}
